"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var common_helpers_1 = require("./common-helpers");
var constants_1 = require("./constants");
var errors_enum_1 = require("./errors.enum");
var http_helper_1 = require("./http-helper");
var secret_type_enum_1 = require("./secret-type.enum");
var debug_1 = __importDefault(require("debug"));
var debugThis = debug_1.default("wechat-work:access-token-class");
var fetching = {}; // TODO: multi server
var AccessToken = /** @class */ (function () {
    function AccessToken(corpId, accessToken, expiresIn, secretType, agentId, config) {
        this.accessToken = accessToken;
        this.expiresIn = expiresIn;
        this.createdAt = Date.now();
        this.expiresAt = this.createdAt + this.expiresIn * 1000;
        this.corpId = corpId;
        this.agentId = agentId;
        this.secretType = secretType;
        this.config = config;
    }
    AccessToken.getInstance = function (config, secretType, agentId) {
        return __awaiter(this, void 0, void 0, function () {
            var key, cachedAccessToken, deSerialized, deSerialized, fetched, err_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        key = AccessToken.getStorageKey(config.corpId, secretType, agentId);
                        debugThis("storage key for access token: " + key);
                        return [4 /*yield*/, config.getFromCacheMethod(key)];
                    case 1:
                        cachedAccessToken = _a.sent();
                        if (cachedAccessToken && cachedAccessToken.content) {
                            if (cachedAccessToken.ttlSeconds) {
                                if (cachedAccessToken.ttlSeconds > constants_1.ACCESS_TOKEN_EXPIRE_CUTOFF) {
                                    deSerialized = AccessToken.deSerialize(cachedAccessToken.content, cachedAccessToken.ttlSeconds);
                                    deSerialized.config = config;
                                    return [2 /*return*/, deSerialized];
                                }
                            }
                            else {
                                deSerialized = AccessToken.deSerialize(cachedAccessToken.content);
                                if (deSerialized.expiresIn > 600) {
                                    deSerialized.config = config;
                                    deSerialized.secretType = secretType;
                                    // internally maintained ttl is valid
                                    return [2 /*return*/, deSerialized];
                                }
                            }
                        }
                        if (!(fetching[key] === true)) return [3 /*break*/, 4];
                        debugThis("waiting for another process to release this key: " + key);
                        return [4 /*yield*/, common_helpers_1.timer(100)];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, AccessToken.getInstance(config, secretType, agentId)];
                    case 3: return [2 /*return*/, _a.sent()];
                    case 4:
                        fetching[key] = true;
                        _a.label = 5;
                    case 5:
                        _a.trys.push([5, 8, , 9]);
                        return [4 /*yield*/, AccessToken.fetch(config.corpId, AccessToken.getSecret(config, secretType, agentId), secretType, agentId)];
                    case 6:
                        fetched = _a.sent();
                        fetched.config = config;
                        return [4 /*yield*/, config.saveToCacheMethod(fetched.storageKey, fetched.serialized, fetched.expiresIn)];
                    case 7:
                        _a.sent();
                        fetching[key] = false;
                        return [2 /*return*/, fetched];
                    case 8:
                        err_1 = _a.sent();
                        fetching[key] = false;
                        throw err_1;
                    case 9: return [2 /*return*/];
                }
            });
        });
    };
    AccessToken.deSerialize = function (serialized, ttlSeconds) {
        if (!serialized) {
            throw Error("serialized access token can not be an empty string.");
        }
        var splited = serialized.split("|");
        if (splited.length !== 4) {
            throw Error("invalid serialized access token.");
        }
        var expiresIn = ttlSeconds
            ? ttlSeconds
            : (parseInt(splited[1], 10) - Date.now()) / 1000;
        return new AccessToken(splited[2].trim(), splited[0].trim(), expiresIn, parseInt(splited[3], 10), splited[4]);
    };
    AccessToken.fetch = function (corpId, secret, secretType, agentId) {
        return __awaiter(this, void 0, void 0, function () {
            var url, resData;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        url = "https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid=" + corpId + "&corpsecret=" + secret;
                        return [4 /*yield*/, http_helper_1.doGet(url)];
                    case 1:
                        resData = (_a.sent());
                        return [2 /*return*/, new AccessToken(corpId, resData.access_token, resData.expires_in, secretType, agentId)];
                }
            });
        });
    };
    AccessToken.getStorageKey = function (corpId, secretType, agentId) {
        // tslint:disable-next-line:no-console
        console.log("secretType: ", secretType);
        var key = "WECHAT_WORK_ACCESS_TOKEN:" + corpId;
        switch (secretType) {
            case secret_type_enum_1.EnumSecretType.Contact:
                key += ":Contact";
                break;
            case secret_type_enum_1.EnumSecretType.ExternalContact:
                key += ":ExternalContact";
                break;
            case secret_type_enum_1.EnumSecretType.Agent:
                if (!agentId) {
                    throw Error(errors_enum_1.EnumErrors.AGENT_ID_SHOULD_PROVIDED);
                }
                key += ":AGENT:" + agentId;
                break;
            default:
                throw Error(errors_enum_1.EnumErrors.SECRET_TYPE_NOT_SUPPORTED);
        }
        return key;
    };
    AccessToken.getSecret = function (config, secretType, agentId) {
        var secret = "";
        switch (secretType) {
            case secret_type_enum_1.EnumSecretType.Contact:
                if (config.contactSecret === undefined) {
                    throw Error(errors_enum_1.EnumErrors.CONTACT_SECRET_SHOULD_PROVIDED);
                }
                secret = config.contactSecret;
                break;
            case secret_type_enum_1.EnumSecretType.ExternalContact:
                if (config.externalContactSecret === undefined) {
                    throw Error(errors_enum_1.EnumErrors.EXTERNAL_CONTACT_SECRET_SHOULD_PROVIDED);
                }
                secret = config.externalContactSecret;
                break;
            case secret_type_enum_1.EnumSecretType.Agent:
                debugThis("desired agentId is " + agentId);
                debugThis(JSON.stringify(config.apps));
                var appsFound = config.apps.filter(function (app) { return app.agentId === agentId; });
                if (appsFound.length === 0) {
                    throw Error(errors_enum_1.EnumErrors.AGENT_NOT_CONFIGURED);
                }
                secret = appsFound[0].agentSecret;
                break;
            default:
                throw Error(errors_enum_1.EnumErrors.SECRET_TYPE_NOT_SUPPORTED);
        }
        return secret;
    };
    AccessToken.prototype.isExpired = function () {
        return this.expiresAt <= Date.now();
    };
    Object.defineProperty(AccessToken.prototype, "storageKey", {
        get: function () {
            return AccessToken.getStorageKey(this.corpId, this.secretType, this.agentId);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AccessToken.prototype, "serialized", {
        get: function () {
            return this.accessToken + "|" + this.expiresAt + "|" + this.corpId + "|" + this.agentId;
        },
        enumerable: true,
        configurable: true
    });
    AccessToken.prototype.ensureNotExpired = function () {
        return __awaiter(this, void 0, void 0, function () {
            var token;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.isExpired()) return [3 /*break*/, 2];
                        if (!this.config) {
                            throw Error(errors_enum_1.EnumErrors.CONFIG_SHOULD_PROVIDED);
                        }
                        return [4 /*yield*/, AccessToken.getInstance(this.config, this.secretType, this.agentId)];
                    case 1:
                        token = _a.sent();
                        this.accessToken = token.accessToken;
                        this.createdAt = token.createdAt;
                        this.expiresAt = token.expiresAt;
                        this.expiresIn = token.expiresIn;
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    return AccessToken;
}());
exports.AccessToken = AccessToken;
//# sourceMappingURL=access-token.class.js.map